package org.jboss.testmdbconcurrency;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.management.ManagementFactory;
import java.util.Arrays;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.Asynchronous;
import javax.ejb.SessionContext;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.management.MBeanServerConnection;
import javax.management.MBeanServerInvocationHandler;
import javax.management.ObjectName;

import org.hornetq.api.jms.HornetQJMSConstants;
import org.hornetq.api.jms.management.JMSQueueControl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Singleton
@Startup
public class MyStartupSingleton {

	private static final Logger log = LoggerFactory.getLogger(MyStartupSingleton.class);

	@Resource(mappedName = "java:/JmsXA")
	private ConnectionFactory connectionFactory;

	@Resource(mappedName = Constants.QUEUE_PATH_AND_NAME)
	private Queue queue;

	@Resource
	private SessionContext context;

	@PostConstruct
	public void onStartup() {
		MyStartupSingleton me = context.getBusinessObject(MyStartupSingleton.class);
		me.runTestAsync();
	}

	@Asynchronous
	public void runTestAsync() {

		try {
			// give time for server to complete startup
			Thread.sleep(10 * 1000);
			// run the test
			runTest();
		} catch (Exception e) {
			log.error("Unexpected error occurred during test", e);
		}
	}

	private void runTest() throws Exception {

		log.info("Running test");

		sendMessages();

		log.info("Sleeping for 10s before logging queue stats");
		Thread.sleep(10 * 1000);

		logQueueStats();
	}

	private void sendMessages() throws Exception {

		MyStartupSingleton me = context.getBusinessObject(MyStartupSingleton.class);

		byte[] oneHundredKBMessageData = getOneHundredKBByteArray();

		// First, send 100 messages with the same group id
		log.info("Sending 100 group1 messages");
		String group1 = "group1";
		for (int i = 0; i < 100; i++) {
			me.sendMessageWithGroupIdInNewTrans(group1, "mygroup1message_" + i, oneHundredKBMessageData);
		}

		log.info("Sending single message for each of group2 to group20");
		for (int i = 2; i <= 20; i++) {
			me.sendMessageWithGroupIdInNewTrans("group" + i, "mygroup" + i + "message_0", oneHundredKBMessageData);
		}

	}

	@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
	public void sendMessageWithGroupIdInNewTrans(String groupId, String messageId, byte[] message) throws Exception {

		Connection conn = connectionFactory.createConnection();

		Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);

		// create a message producer
		MessageProducer producer = session.createProducer(queue);

		BufferedInputStream bufferedInput = null;
		InputStream stringByteInputStream = null;

		Message filteredMessage = session.createBytesMessage();

		stringByteInputStream = new ByteArrayInputStream(message);
		bufferedInput = new BufferedInputStream(stringByteInputStream);
		filteredMessage.setObjectProperty(HornetQJMSConstants.JMS_HORNETQ_INPUT_STREAM, bufferedInput);

		filteredMessage.setStringProperty("JMSXGroupID", groupId);

		filteredMessage.setStringProperty(Constants.MESSAGE_NAME_PROPERTY, messageId);

		try {

			producer.send(filteredMessage);

		} finally {

			// ensure our streams are closed
			if (bufferedInput != null) {
				try {
					bufferedInput.close();
				} catch (IOException ioe) {
					// don't care
					log.debug("Could not close buffered input stream. Message: {}", ioe.getMessage());
				}
			}

			if (stringByteInputStream != null) {
				try {
					stringByteInputStream.close();
				} catch (IOException ioe) {
					// don't care
					log.debug("Could not close input stream. Message: {}", ioe.getMessage());
				}
			}

			conn.close();
		}

	}

	private byte[] getOneHundredKBByteArray() {
		byte[] bytes = new byte[100 * 1024];
		Arrays.fill(bytes, (byte) 8);
		return bytes;
	}

	private void logQueueStats() throws Exception {

		MBeanServerConnection mbsc = ManagementFactory.getPlatformMBeanServer();
		String objectName = "org.hornetq:module=JMS,type=Queue,name=\"" + Constants.QUEUE_NAME + "\"";
		ObjectName queueObjectName = new ObjectName(objectName);
		JMSQueueControl queueControl = MBeanServerInvocationHandler.newProxyInstance(mbsc, queueObjectName, JMSQueueControl.class, false);

		int consumerCount = queueControl.getConsumerCount();
		int deliveringCount = queueControl.getDeliveringCount();
		long messageCount = queueControl.getMessageCount();
		long messagesAdded = queueControl.getMessagesAdded();

		log.info("Queue properties: \nconsumerCount={}, \ndeliveringCount={}, \nmessageCount={}, \nmessagesAdded={}", new Object[] { consumerCount,
				deliveringCount, messageCount, messagesAdded });

	}

}
